package org.ow2.frascati.factory.api;

import java.net.URL;
import java.net.URLClassLoader;

import org.objectweb.fractal.api.Component;

public interface FrascatiFactory {
  
  /**
   * Create an SCA assembly instance from composite file
   * 
   * @param composite
   *          The composite file located into the current class path, on the
   *          local file system, or at an URL
   * 
   * @return the SCA composite component or null if an error occurs.
   * @throws Exception 
   */
  
  public Component getComposite(String composite) throws Exception;
  
  /**
   * Load jars and create an SCA assembly instance from composite file
   * 
   * @param composite
   *          - The composite file located into the current class path, on the
   *          local file system, or at an URL
   * @param libs
   *          - Libraries required to load this composite
   * 
   * @return the SCA composite component
   * @throws Exception 
   */
  
  public Component getComposite(String composite, URL[] libs) throws Exception;
  
  /**
   * Set the URLClassLoader used by the Factory
   * 
   * @param classloader
   *          - URLClassLoader to be used by the Factory
   */
  
  public void setClassLoader(URLClassLoader classloader);
  
  /**
   * Get the ClassLoader used by the Factory
   * 
   * @return - ClassLoader currently used by the Factory
   */
  
  public ClassLoader getClassLoader();
  
  /**
   * Close a SCA composite.
   * 
   * @param composite
   *          The name of the SCA composite to close.
   * @throws Exception
   */
  
  public void close(Component composite) throws Exception;

}
