/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.core.dispatcher;

import java.net.URL;

import org.objectweb.fractal.fraclet.annotations.Interface;
import org.objectweb.fractal.api.Component;
import org.ow2.frascati.factory.core.instance.InstantiationException;
import org.ow2.frascati.factory.core.instance.NotSupportedException;

/**
 * Manage Assembly Factory instantiation process
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 */

@Interface(name = "load")
public interface Manager {

  /**
   * Create tinfi composite instance from an SCA composite file
   * 
   * @param composite
   *          - name of the composite file to load, can be a resource for the
   *          current class path, a local file or an URL
   * @return the resulting SCA composite instance
   * @throws InstantiationException 
   * @throws NotSupportedException 
   */

  public Object loadComposite(String composite) throws NotSupportedException, InstantiationException;

  /**
   * Load jars into current class loader
   * 
   * @param urls
   *          - list of jar location
   * @return the ClassLoader used by the Factory
   */

  public ClassLoader loadJars(URL[] urls);

  /**
   * Load jars and create an SCA assembly instance from composite file
   * 
   * @param composite
   *          - name of the composite file to load, can be a resource for the
   *          current class path, a local file or an URL
   * @param libs
   *          - list of jar location
   * @return the resulting SCA composite instance
   * @throws InstantiationException 
   * @throws NotSupportedException 
   */

  public Component getComposite(String composite, URL[] libs) throws NotSupportedException, InstantiationException;

  /**
   * Load a contribution zip archive
   * 
   * @param contribution - name of the contribution file to load
   * @return A set of composites which has been loaded
   * @throws InstantiationException 
   * @throws NotSupportedException 
   * @since 1.1
   */

  public Component [] getContribution(String contribution) throws NotSupportedException, InstantiationException;


}
