/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.core.instance.binding;

import org.eclipse.stp.sca.BaseReference;
import org.eclipse.stp.sca.BaseService;
import org.eclipse.stp.sca.Reference;
import org.eclipse.stp.sca.Service;

/**
 * Utility class for binding creation
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 */

public class ScaBindingUtil {

  public static final String DEFAULT_SERVICE_NAME = "service";

  public static final String DEFAULT_REFERENCE_NAME = "reference";

  /**
   * Get Service name, return default name if service name is not defined
   * 
   * @param service -
   *          The Service element
   * @return Service Name
   */

  public static String getServiceName(BaseService service) {
    return (service.getName() != null) ? service.getName()
        : DEFAULT_SERVICE_NAME;
  }

  /**
   * Get Reference name, return default name if reference name is not defined
   * 
   * @param reference -
   *          The Reference element
   * @return Reference name
   */

  public static String getReferenceName(BaseReference reference) {
    return (reference.getName() != null) ? reference.getName()
        : DEFAULT_REFERENCE_NAME;
  }

  /**
   * Get promoted Component and Service name, return default name if service
   * name is not defined
   * 
   * @param service -
   *          The Service element
   * @return A string array composed of the component name and service name
   */

  public static String[] getPromote(Service service) {
    String promoted = service.getPromote();
    String promotes[] = promoted.split("/");

    if (promotes.length == 2)
      return promotes;
    else
      return new String[] { promotes[0], DEFAULT_SERVICE_NAME };
  }

  /**
   * Get promoted Component and Reference name, return default name if reference
   * name is not defined
   * 
   * @param reference -
   *          The Reference element
   * @return A string array composed of the component name and reference name
   */

  public static String[] getPromote(Reference reference) {
    String promoted = reference.getPromote();
    String promotes[] = promoted.split("/");

    if (promotes.length == 2)
      return promotes;
    else
      return new String[] { promotes[0], DEFAULT_REFERENCE_NAME };
  }

}
