/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.core.instance.component;

import org.eclipse.stp.sca.Component;
import org.eclipse.stp.sca.Composite;
import org.objectweb.fractal.fraclet.annotations.Interface;
import org.ow2.frascati.factory.core.instance.InstantiationException;
import org.ow2.frascati.factory.core.instance.NotSupportedException;
import org.ow2.frascati.factory.core.instance.binding.BindingException;
import org.ow2.frascati.factory.core.instance.implementation.ImplementationException;
import org.ow2.frascati.factory.core.instance.interfaces.InterfaceException;

/**
 * Create components instances
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 */

@Interface(name = "components")
public interface ScaComponent {

  /**
   * Create a fractal component according to provided component definition
   * 
   * @param component - Provided component definition
   * @return Resulting fractal component
   * @throws ImplementationException 
   * @throws NotSupportedException 
   * @throws BindingException 
   * @throws InterfaceException 
   * @throws InstantiationException 
   */
  
  public org.objectweb.fractal.api.Component create(Component component) throws NotSupportedException, InstantiationException;
  
  /**
   * Create a fractal component according to provided composite definition
   * 
   * @param assembly
   * @return Resulting fractal component
   * @throws ImplementationException 
   * @throws NotSupportedException 
   * @throws BindingException 
   * @throws InterfaceException 
   * @throws InstantiationException 
   * @throws  
   */
  
  public org.objectweb.fractal.api.Component create(Composite assembly) throws NotSupportedException, InstantiationException;

}
