/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.core.instance.implementation;

import org.eclipse.stp.sca.Implementation;
import org.eclipse.stp.sca.SCAImplementation;
import org.eclipse.stp.sca.ScaPackage;
import org.eclipse.stp.sca.domainmodel.frascati.FrascatiPackage;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.fraclet.types.Contingency;
import org.ow2.frascati.factory.core.dispatcher.Manager;
import org.ow2.frascati.factory.core.domain.DomainException;
import org.ow2.frascati.factory.core.domain.api.CompositeDomain;
import org.ow2.frascati.factory.core.instance.InstantiationException;
import org.ow2.frascati.factory.core.instance.NotSupportedException;
import org.ow2.frascati.factory.core.parser.util.ScaModelHelper;

/**
 * Create SCA component instances implemented as SCA composite
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 */

@org.objectweb.fractal.fraclet.annotations.Component
public class ScaCompositeImpl implements ScaImplementation {

  /**
   * The implementation type calculated with Object Eclass
   */

  public static String IMPLEMENTATION_TYPE = ScaModelHelper
      .getID(ScaPackage.Literals.SCA_IMPLEMENTATION);

  /**
   * Logger for this class
   */

  java.util.logging.Logger log = java.util.logging.Logger
      .getLogger(ScaCompositeImpl.class.getCanonicalName());

  /**
   * The assembly factory dispatcher component required for recursive calls
   */

  @Requires(name = "loader")
  Manager dispatcher;

  /**
   * Reference on composite domain, used to add composite implementation to
   * domain at runtime
   */

  @Requires(contingency = Contingency.OPTIONAL)
  CompositeDomain domain;

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.instance.implementation.ScaImplementation
   * #create( org.eclipse.stp.sca.Implementation,
   * org.objectweb.fractal.api.type.ComponentType)
   */

  public Component create(Implementation implementation,
      ComponentType componentType) throws ImplementationException {

    // Retrieve composite file name
    SCAImplementation ScaImpl = (SCAImplementation) implementation;
    String compositeName = ScaImpl.getName().getLocalPart();

    // Retrieve file URI if prefix is present
    if (!ScaImpl.getName().getPrefix().equals("")) {
      String prefixUri = ScaImpl.getName().getNamespaceURI();

      // Check if namespace uri ends with "/"
      if (!prefixUri.endsWith("/"))
        prefixUri = prefixUri + "/";

      // Concatenate URI string
      compositeName = prefixUri + compositeName;
    }

    Component component = null;

    try {

      // Test if domain is binded
      if (domain != null) {
        // Check if this component has already been loaded
        domain.get(compositeName);

        if (component == null) {

          // Load this composite component
          component = (Component) dispatcher.loadComposite(compositeName);
          // Add composite component into cache
          domain.add(component);
        }
        
      } else
        // When domain not used (only generating components code)
        component = (Component) dispatcher.loadComposite(compositeName);

    } catch (DomainException e) {
      throw new ImplementationException(
          "Error when using domain component with composite implementation plugin",
          e);
    } catch (Exception e) {
      throw new ImplementationException(
          "Error while loading composite implementation '" + compositeName
              + "'", e);
    }

    return component;

  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.instance.implementation.ScaImplementation#
   * getImplementationID()
   */

  public String getImplementationID() {
    return IMPLEMENTATION_TYPE;
  }

}
