/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.core.instance.implementation;

import org.eclipse.stp.sca.Implementation;
import org.eclipse.stp.sca.JavaImplementation;
import org.eclipse.stp.sca.ScaPackage;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.ow2.frascati.factory.core.instance.runtime.RuntimeFactory;
import org.ow2.frascati.factory.core.parser.util.ScaModelHelper;
import org.ow2.frascati.factory.core.instance.InstantiationException;

/**
 * Create SCA component instances implemented in Java language
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 */

@org.objectweb.fractal.fraclet.annotations.Component
public class ScaJavaImpl implements ScaImplementation {

  /**
   * The implementation type calculated with Object Eclass
   */

  public static String IMPLEMENTATION_TYPE = ScaModelHelper.getID(ScaPackage.Literals.JAVA_IMPLEMENTATION);

  /**
   * Logger for this class
   */

  java.util.logging.Logger log = java.util.logging.Logger
      .getLogger(ScaJavaImpl.class.getCanonicalName());

  /**
   * The required runtime
   */

  @Requires(name = "runtime")
  RuntimeFactory runtime;

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.instance.implementation.ScaImplementation
   * #create( org.eclipse.stp.sca.Implementation,
   * org.objectweb.fractal.api.type.ComponentType)
   */

  public Component create(Implementation implementation,
      ComponentType componentType) throws ImplementationException {

    Component component = null;
    JavaImplementation javaImp = (JavaImplementation) implementation;

    try {

      log.finer("Create java component for implementation "
          + javaImp.getClass_());

      // get instance of an SCA primitive component for this
      // implementation
      component = runtime.newInstance(componentType, new Object[] {
          runtime.getClassLoader(), "scaPrimitive" }, javaImp.getClass_());

      return component;

    } catch (InstantiationException e) {

      // Test if java class is present in current classloader
      try {
        runtime.getClassLoader().loadClass(javaImp.getClass_());
      } catch (ClassNotFoundException e1) {
        throw new ImplementationException("Class " + javaImp.getClass_()
            + " may not be loaded by the factory", e);
      }

      throw new ImplementationException(
          "Error while creating Java component instance : "
              + javaImp.getClass_(), e);
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.instance.implementation.ScaImplementation#
   * getImplementationID()
   */

  public String getImplementationID() {
    return IMPLEMENTATION_TYPE;
  }

}
