/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 * 
 * Contributor(s): Nicolas Dolet
 */
package org.ow2.frascati.factory.core.instance.intent;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.ow2.frascati.factory.core.dispatcher.Manager;
import org.ow2.frascati.factory.core.domain.api.CompositeDomain;
import org.ow2.frascati.factory.core.instance.intent.api.Cache;

/**
 * Cache {@link Component} containing loaded intent components
 * 
 * @author <a href="mailto:nicolas.dolet@inria.fr">Nicolas Dolet</a>
 * @version $Revision$
 */
@org.objectweb.fractal.fraclet.annotations.Component
public class CacheImpl implements Cache {

  // Reference on the load interface provided by the Factory
  @Requires(name = "loader")
  Manager dispatcher;

  @Requires(name = "domain-management")
  CompositeDomain domain;

  // --------------------------------------------------------------------------
  // Implementation of the Cache interface
  // --------------------------------------------------------------------------

  /**
   * @throws IntentException
   * @see org.ow2.frascati.factory.core.instance.intent.api.Cache#getFromCache(String)
   */
  public Component getFromCache(String intentName) throws IntentException {
    try {

      Component res = domain.get(intentName);
      
      if (res == null) {
        // Load this intent component
        res = (Component) dispatcher.loadComposite(intentName);
        // Add intent component into cache
        domain.add(res);

      }

      return res;
      
    } catch (Exception e) {
      throw new IntentException("Error while loading intent '" + intentName
          + "'", e);
    }

  }
}
