/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.core.instance.interfaces;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.eclipse.stp.sca.Interface;
import org.eclipse.stp.sca.ScaPackage;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.fraclet.annotations.Lifecycle;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.fraclet.types.Cardinality;
import org.objectweb.fractal.fraclet.types.Policy;
import org.objectweb.fractal.fraclet.types.Step;
import org.ow2.frascati.factory.core.instance.NotSupportedException;
import org.ow2.frascati.factory.core.parser.util.ScaModelHelper;

@org.objectweb.fractal.fraclet.annotations.Component
public class ScaInterfaceSwitch implements ScaInterface {
  
  /**
   * The interface type calculated with Object Eclass
   */

  public static String INTERFACE_TYPE = ScaModelHelper.getID(ScaPackage.Literals.INTERFACE);

  /**
   * Logger for this class
   */

  java.util.logging.Logger log = java.util.logging.Logger
      .getLogger(ScaInterfaceSwitch.class.getCanonicalName());

  @Requires(name = "plugins", cardinality = Cardinality.COLLECTION)
  private Map<String, ScaInterface> interfacePlugins = new HashMap<String, ScaInterface>();

  // Map providing direct access to interface plugins
  private Map<String, ScaInterface> interfaceIDMap = new HashMap<String, ScaInterface>();

  /**
   * Interface Switch initializer, Build map for accessing interface plugins
   * according to the Interface ID
   */

  @Lifecycle(step = Step.START, policy = Policy.ASYNCHRONOUS)
  public void initialize() {
    // construct map for accessing interface plugins using EMF Classifier ID
    Set<String> keys = interfacePlugins.keySet();
    for (String key : keys) {
      interfaceIDMap.put(interfacePlugins.get(key).getInterfaceID(),
          interfacePlugins.get(key));
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.instance.interfaces.ScaInterface#create(org
   * .eclipse.stp.sca.Interface)
   */

  // Choose the appropriate Interface plug-in according to interface type
  // throw NotSupportedException if any suitable plug-in can be found

  public InterfaceType create(Interface scaInterface)
      throws NotSupportedException, InterfaceException {

    // check if an Interface has been given
    if (scaInterface != null) {
      // retrieve registered interface plug-in from cache
      String id = ScaModelHelper.getID(scaInterface);
      ScaInterface itfPlugin = interfaceIDMap.get(id);

      if (itfPlugin != null) {
        // delegates call to interface plug-in
        return itfPlugin.create(scaInterface);
      } else
        // no suitable plug-in found for this interface type
        throw new NotSupportedException("Interface type not supported : "
            + id);
    } else {
      throw new InterfaceException("Unknown interface for Reference/Service");
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.instance.interfaces.ScaInterface#getInterfaceID
   * ()
   */

  public String getInterfaceID() {
    return INTERFACE_TYPE;
  }

}
