/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.core.instance.interfaces;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.stp.sca.BaseReference;
import org.eclipse.stp.sca.BaseService;
import org.eclipse.stp.sca.Interface;
import org.eclipse.stp.sca.JavaInterface;
import org.eclipse.stp.sca.Multiplicity;
import org.eclipse.stp.sca.ScaPackage;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.api.type.TypeFactory;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.ow2.frascati.factory.core.instance.runtime.RuntimeFactory;
import org.ow2.frascati.factory.core.parser.util.ScaModelHelper;
import org.ow2.frascati.factory.core.instance.InstantiationException;

/**
 * Create interface and callback interface types from java interfaces
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 */

@org.objectweb.fractal.fraclet.annotations.Component
public class ScaJavaInterfaceImpl implements ScaInterface {

  /**
   * The interface type calculated with Object Eclass
   */

  public static String INTERFACE_TYPE = ScaModelHelper.getID(ScaPackage.Literals.JAVA_INTERFACE);


  /**
   * Logger for this class
   */

  java.util.logging.Logger log = java.util.logging.Logger
      .getLogger(ScaJavaInterfaceImpl.class.getCanonicalName());

  /**
   * The required runtime
   */

  @Requires(name = "runtime")
  RuntimeFactory runtime;

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.instance.JavaInterfaceBuilder#create(org.
   * eclipse. stp.sca.BaseService)
   */

  public InterfaceType create(Interface scaInterface) throws InterfaceException {

    // Cast the abstract model interface as a Java Interface
    JavaInterface itf = (JavaInterface) scaInterface;

    // Get the Service or Reference Element for this Interface
    EObject obj = itf.eContainer();

    // Name for this interface
    String interfaceName;

    // Corresponding java interface class
    String interfaceClass = itf.getInterface();

    // indicates if component interface is mandatory or optional
    boolean contingency = TypeFactory.MANDATORY;

    // indicates if component interface is single or collection
    boolean cardinality = TypeFactory.SINGLE;

    // indicates if it client or service interface
    boolean isClientItf;
    
    // Test if java interface is present in current classloader
    try {
      runtime.getClassLoader().loadClass(interfaceClass);
    } catch (ClassNotFoundException ei) {
      throw new InterfaceException("Interface " + interfaceClass
          + " may not be loaded by the factory", ei);
    }
    
    // Retrieve interface parameter
    if (obj instanceof BaseService) {
      BaseService s = (BaseService) obj;
      interfaceName = s.getName();
      isClientItf = TypeFactory.SERVER;
    }
    // obj instanceof BaseReference
    else {
      BaseReference r = (BaseReference) obj;
      interfaceName = r.getName();
      isClientItf = TypeFactory.CLIENT;

      // change reference multiplicity if necessary
      Multiplicity multiplicity = r.getMultiplicity();

      if (multiplicity != null) {
        contingency = ScaMultiplicity.getcontingency(multiplicity.getLiteral());
        cardinality = ScaMultiplicity.getcardinality(multiplicity.getLiteral());
      }
    }

    // Try to create interface type
      log.finer("Create interface type " + interfaceClass + " for "
          + interfaceName + " with contingency = " + contingency
          + " and cardinality = " + cardinality);
      try {
      return runtime.createInterfaceType(interfaceName, interfaceClass,
          isClientItf, contingency, cardinality);
    } catch (InstantiationException e) {
      log.warning("Could not create interface for " + interfaceClass
          + " service " + interfaceName);
      e.printStackTrace();
    }
    
    // This error should never occur
    throw new InterfaceException("Could not create java interface");

  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.instance.interfaces.ScaInterface#getInterfaceID
   * ()
   */

  public String getInterfaceID() {
    return INTERFACE_TYPE;
  }

}
