/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 * 
 * Contributor(s): Nicolas Dolet
 */

package org.ow2.frascati.factory.core.parser.util;

import java.lang.reflect.Method;
import java.util.Map;

import org.eclipse.stp.sca.Component;
import org.eclipse.stp.sca.ComponentReference;
import org.eclipse.stp.sca.ComponentService;
import org.eclipse.stp.sca.Composite;
import org.eclipse.stp.sca.Implementation;
import org.eclipse.stp.sca.Interface;
import org.eclipse.stp.sca.JavaImplementation;
import org.eclipse.stp.sca.JavaInterface;
import org.eclipse.stp.sca.ScaPackage;
import org.eclipse.stp.sca.introspection.metadata.MetaData;
import org.osoa.sca.annotations.Callback;
import org.ow2.frascati.factory.core.instance.runtime.RuntimeFactory;

public class CallbackInterfaceResolver implements Resolver {

  /**
   * Logger for this class
   */

  java.util.logging.Logger log = java.util.logging.Logger
      .getLogger(CallbackInterfaceResolver.class.getCanonicalName());

  /**
   * Reference on runtime component
   */

  protected RuntimeFactory runtime;

  public CallbackInterfaceResolver(RuntimeFactory runtime) {
    this.runtime = runtime;
  }

  /**
   * Resolve CallBack for components references/services
   * 
   * @param composite
   * @return resulting composite
   */

  public Composite resolve(Composite composite) {

    for (Component component : composite.getComponent()) {

      // Get component implementation
      Implementation imp = component.getImplementation();

      // Check if implementation is Java
      if (imp.eClass().getClassifierID() == ScaPackage.JAVA_IMPLEMENTATION) {

        JavaImplementation javaImpl = (JavaImplementation) imp;
        try {

          // Retrieve Metadata for this component implementation
          Class<?> cl = runtime.getClassLoader()
              .loadClass(javaImpl.getClass_());
          MetaData<?> ccmd = new MetaData(cl);

          // Resolve CallBack references/services
          resolveItf(component, ccmd);

        } catch (ClassNotFoundException e) {
          log.fine("Could not get Meta Data from : " + javaImpl.getClass_());
        } catch (Exception e) {
          throw new Error(e);
        }
      }

    }

    // resulting composite model
    return composite;

  }

  private void resolveItf(Component component, MetaData<?> ccmd) {

    for (ComponentService service : component.getService()) {
      if (service.getCallback() != null) {
        Map<String, Method> setters = ccmd.getAllUnAnnotatedSetterMethods();
        Method m = setters.get(service.getName());
        if (m != null) {
          log.fine("add Callbacks on method" + m.getName());
          ccmd.addMetaData(m, org.osoa.sca.annotations.Callback.class);
        }
        resolveJavaItf(service.getInterface());
      }
    }

    for (ComponentReference reference : component.getReference()) {
      if (reference.getCallback() != null) {
        Map<String, Method> setters = ccmd.getAllUnAnnotatedSetterMethods();
        Method m = setters.get(reference.getName());
        if (m != null) {
          log.fine("add Callbacks on method" + m.getName());
          ccmd.addMetaData(m, org.osoa.sca.annotations.Callback.class);
        }
        resolveJavaItf(reference.getInterface());
      }
    }
  }

  private void resolveJavaItf(Interface itf) {
    if (itf.eClass().getClassifierID() == ScaPackage.JAVA_INTERFACE) {

      try {

        JavaInterface javaInterface = (JavaInterface) itf;

        if (javaInterface.getCallbackInterface() != null) {

          // Retrieve Metadata for this interface
          Class<?> cl = runtime.getClassLoader().loadClass(
              javaInterface.getInterface());
          MetaData<?> metadata = new MetaData(cl);

          Callback callback = metadata.getAnnotation(Callback.class);

          if (callback == null) {
            //callback = metadata.addMetaData(c, md);
          }

        }

      } catch (ClassNotFoundException e) {
        // Nothing to do
        // e.printStackTrace();
      }
    }
  }
}
