/***
 * OW2 FraSCAti explorer
 * Copyright (C) 2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Christophe Demarey
 * 
 * Contributor(s):
 */
package org.ow2.frascati.explorer.fscript;

import java.util.logging.Logger;

import org.ow2.frascati.explorer.FrascatiExplorerLauncher;
import org.ow2.frascati.fscript.FraSCAtiScript;

/**
 * FraSCAtiScript Explorer launcher.
 * It allows to:
 * <ul>
 * <li>instanciate the FraSCAti runtime,</li>
 * <li>instanciate the FScript runtime,</li>
 * <li>instanciate the explorer architecture,</li>
 * <li>eventually load SCA composites,</li>
 * <li>and show the FraSCAti explorer GUI.</li>
 * 
 * @author Christophe Demarey.
 */
public class FrascatiScriptExplorerLauncher 
     extends FrascatiExplorerLauncher {
  /** Default Explorer ADL file */
  private static final String DEFAULT_EXPLORER_ADL = "org.ow2.frascati.explorer.fscript.FrascatiScriptExplorer";
  /** The logger */
  protected static final Logger log = Logger.getLogger("org.ow2.frascati.explorer.FrascatiExplorerLauncher");
   
  /**
   * Default Constructor.
   */
  public FrascatiScriptExplorerLauncher() {
    String adlFile = System.getProperty("frascati.explorer.adl");

    if (adlFile == null) adlFile = DEFAULT_EXPLORER_ADL;
    FraSCAtiScript.createSingleton(adlFile);
    this.explorer = FraSCAtiScript.getSingleton().getRootComponent();
  }

  /**
   * The bootstrap method.
   * 
   * @param args - program arguments.
   */
  public static void main(String[] args) {
    FrascatiScriptExplorerLauncher launcher = new FrascatiScriptExplorerLauncher();
    
    launcher.loadComposites(args);   
    launcher.showGui();
  }
}
