/***
 * OW2 FraSCAti fscript
 * Copyright (C) 2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Christophe Demarey
 *
 * Contributor(s):
 */
package org.ow2.frascati.explorer.fscript.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;
import javax.swing.text.StyledDocument;

import org.objectweb.fractal.fscript.FScriptException;
import org.ow2.frascati.fscript.FraSCAtiScript;

/**
 * A GUI console allowing to execute FraSCAtiScript queries / commands.
 * 
 * @author Christophe Demarey.
 */
public class Console extends JFrame {
  private static final String title = "FraSCAti Script console";
  private static final String prompt = "FraSCAtiScript> ";
  
  /** Graphical components. */
  private javax.swing.JTextField input = new JTextField(40);
  private javax.swing.JScrollPane outputScrollPane = new JScrollPane();
  private javax.swing.JTextPane output = new JTextPane();
  /** the output document */
  private StyledDocument doc = output.getStyledDocument();

  /**
   * Default constructor.
   */
  public Console() {
      super(title);
      this.setLayout( new BoxLayout(this.getContentPane(), BoxLayout.Y_AXIS) );
      addStylesToDocument( output.getStyledDocument() );
      addComponents();
      welcome();
      this.pack();
      this.setVisible(true);
  }
  
  /**
   * Display Welcome information for the FraSCAti Script console.
   */
  protected void welcome() {
      executeCommand("$root");
  }
  
  /**
   * Initialize components and add tem to the GUI.
   */
  protected void addComponents() {
      outputScrollPane.setViewportView(output);
      output.setBackground(new Color(255, 255, 204));
      output.setFocusable(false);
      outputScrollPane.setPreferredSize(new Dimension(500,300));
      
      input.setMinimumSize( new Dimension(4,19) );
      input.setMaximumSize( new Dimension(800,19) );
      input.addActionListener( 
        new ActionListener() {        
          public void actionPerformed(ActionEvent e) {
            String cmd = input.getText();
            if ( cmd.trim().length() > 0 ) {
                executeCommand( input.getText() );
            } else {
                // Display the command on the output console
                try {
                  doc.insertString( doc.getLength(), prompt+"\n", doc.getStyle("prompt") );
                } catch (BadLocationException e2) {
                    e2.printStackTrace(); // Should not happen
                }
                input.setText("");
            }
          }
        } );
      
      this.getContentPane().add(outputScrollPane);
      this.getContentPane().add(input);
  }
  
  /**
   * Define some basic style for the output JTextPane.
   * @param doc - Define styles on the document doc.
   */
  protected void addStylesToDocument(StyledDocument doc) {
      Style def = StyleContext.getDefaultStyleContext().
                      getStyle(StyleContext.DEFAULT_STYLE);

      Style regular = doc.addStyle("regular", def);
      StyleConstants.setFontFamily(def, "SansSerif");

      Style s = doc.addStyle("italic", regular);
      StyleConstants.setItalic(s, true);

      s = doc.addStyle("bold", regular);
      StyleConstants.setBold(s, true);

      s = doc.addStyle("small", regular);
      StyleConstants.setFontSize(s, 10);

      s = doc.addStyle("prompt", regular);
      StyleConstants.setForeground(s, Color.GREEN);
      
      s = doc.addStyle("command", regular);
      StyleConstants.setForeground(s, Color.BLUE);      
  }
  
  /**
   * Execute a FraSCAti Script command and update the display (input and output components).
   * @param cmd - The command to execute.
   */
  protected void executeCommand(String cmd) {
      // Display the command on the output console
      try {
        doc.insertString( doc.getLength(), prompt, doc.getStyle("prompt") );
        doc.insertString( doc.getLength(), cmd +"\n", doc.getStyle("command") );
      } catch (BadLocationException e) {
          e.printStackTrace(); // Should not happen
      }
      input.setText("");
      
      // Display the command result
      try {
          try {
              String result = FraSCAtiScript.getSingleton().execute(cmd).toString();
              doc.insertString( doc.getLength(), 
                                result + "\n",
                                doc.getStyle("regular") );
          } catch (FScriptException e2) {
              doc.insertString( doc.getLength(), 
                                "Error while executing FraSCAti Script command\n",
                                doc.getStyle("bold") );
              doc.insertString( doc.getLength(), 
                                e2.getMessage()+"\n",
                                doc.getStyle("small") );
              e2.printStackTrace();
          }
      } catch (BadLocationException e) {
          e.printStackTrace(); // Should not happen
      }
      
     // outputScrollPane.s
  }
}
