/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.runtime.instance;

import java.io.IOException;

import org.objectweb.fractal.api.Type;
import org.objectweb.fractal.fraclet.annotations.Component;
import org.objectweb.fractal.juliac.JuliacException;
import org.ow2.frascati.factory.core.instance.runtime.RuntimeFactory;
import org.ow2.frascati.factory.generate.runtime.RuntimeGenerate;
import org.ow2.frascati.factory.generate.runtime.TinfiGenerate;
import org.ow2.frascati.factory.core.instance.InstantiationException;

/**
 * GenericFactory and TypeFactory component service
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 */

@Component
public class Runtime extends TinfiGenerate implements RuntimeGenerate,
    RuntimeFactory {

  /* (non-Javadoc)
   * @see org.ow2.frascati.factory.core.instance.runtime.TinfiRuntime#newInstance(org.objectweb.fractal.api.Type, java.lang.Object, java.lang.Object)
   */
  
  @Override
  public org.objectweb.fractal.api.Component newInstance(Type arg0,
      Object arg1, Object arg2) throws InstantiationException {

      try {
        Object controllerDesc = arg1;

        if (arg1.getClass().isArray()) {
          Object[] array = (Object[]) arg1;
          controllerDesc = array[1];
        }
  
        jc.getFCSourceCodeGenerator(arg0, controllerDesc, arg2).generate(arg0, controllerDesc, arg2);
        compile();

        return gf.newFcInstance(arg0, arg1, arg2);
        
      } catch (IllegalArgumentException e) {
        throw new InstantiationException("Could not get tinfi source code generator",e);
      } catch (IOException e) {
        throw new InstantiationException("Cannot generate tinfi component code",e);
      } catch (org.objectweb.fractal.api.factory.InstantiationException e) {
        throw new InstantiationException("Could not create tinfi component instance",e);
      } catch (JuliacException e) {
        throw new InstantiationException("Cannot compile generated code",e);
      }

    

    //return null;

  }

}
