/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.control.content;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;

/**
 * This class represents an injection point which is implemented as a field.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @since 1.1.2
 */
public class InjectionPointFieldImpl<T extends Annotation>
extends InjectionPointImpl<T> {

    private Field field;

    public InjectionPointFieldImpl( Field field, T annot ) {
    	super(annot);
    	this.field = field;
        field.setAccessible(true);  // Enable access to private fields
    }
    
    public void set( Object target, Object value )
    throws IllegalAccessException {
        field.set(target,value);
    }
    
    public Object get( Object target ) throws IllegalAccessException {
        Object value = field.get(target);
        return value;
    }

    /**
     * Return the type of the injection point.
     */
    public Class<?> getType() {
    	return field.getType();
    }
}
