/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.control.content;

import java.lang.annotation.Annotation;
import java.lang.reflect.AccessibleObject;
import java.lang.reflect.Field;
import java.lang.reflect.Method;

import org.ow2.frascati.tinfi.reflect.Util;

/**
 * Default abastract implementation of the {@link InjectionPoint} interface.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @since 1.1.2
 */
public abstract class InjectionPointImpl<T extends Annotation>
implements InjectionPoint<T> {

    private T annot;

    public InjectionPointImpl( T annot ) {
    	this.annot = annot;
    }
	
    /**
     * Return the annotation associated with the injection point.
     */
    public T getAnnotation() {
    	return annot;
    }

	/**
	 * Factory method for injection points.
	 * 
	 * @param <T>    the annotation type
	 * @param ao     a setter method or field
	 * @param annot  the annotation
	 * @return       the corresponding injection point
	 */
	public static <T extends Annotation> InjectionPoint<T> getInjectionPoint(
		AccessibleObject ao, T annot ) {
		
		InjectionPoint<T> ip = null;
		
		if( ao instanceof Method ) {
	        Method setter = (Method) ao;
	        try {
				Method getter = Util.getGetterForSetter(setter);
	            ip = new InjectionPointMethodImpl<T>(setter,getter,annot);
			}
	        catch (NoSuchMethodException e) {
	            ip = new InjectionPointMethodImpl<T>(setter,annot);
			}
	    }
	    else {
	        Field field = (Field) ao;
	        ip = new InjectionPointFieldImpl<T>(field,annot);
	        return ip;
	    }
	
		return ip;
	}    
}
