/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2007-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.control.content.scope;

import java.util.Stack;

import org.ow2.frascati.tinfi.osoa.ConversationImpl;

/**
 * This class manages a thread-local stack of conversations.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 */
public class ConversationManager {

    /**
     * Return the singleton instance of this class.
     * 
     * @since 1.1.1
     */
    public static ConversationManager get() {
        Stack<ConversationImpl> stack = tl.get();
        if( stack == null ) {
            stack = new Stack<ConversationImpl>();
            tl.set(stack);
        }
        return instance;
    }
    private static ConversationManager instance = new ConversationManager();
    
    /**
     * The thread-local stack of conversations. The {@link ConversationImpl}
     * class is used instead of the {@link org.osoa.sca.Conversation} interface
     * since some public methods from the class which are not defined in the
     * interface are needed by code which invokes the current class.
     * 
     * @since 1.1.1
     */
    private static ThreadLocal<Stack<ConversationImpl>> tl =
        new ThreadLocal<Stack<ConversationImpl>>();
    
    /**
     * @since 1.1.1
     */
    public void push( ConversationImpl conv ) {
        Stack<ConversationImpl> stack = tl.get();
        stack.push(conv);
    }
    
    /**
     * @since 1.1.1
     */
    public ConversationImpl peek() {
        Stack<ConversationImpl> stack = tl.get();
        ConversationImpl conv = stack.peek();
        return conv;
    }

    /**
     * @since 1.1.1
     */
    public ConversationImpl pop() {
        Stack<ConversationImpl> stack = tl.get();
        ConversationImpl conv = stack.pop();
        return conv;
    }
}
