/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2007-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.osoa;

import org.osoa.sca.NoRegisteredCallbackException;
import org.osoa.sca.ServiceReference;

/**
 * Implementation of a service reference.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 */
public class ServiceReferenceImpl<B> extends CallableReferenceImpl<B>
implements ServiceReference<B> {
    
    public ServiceReferenceImpl( Class<B> businessInterface, B service ) {
        super(businessInterface,service);
    }
    
    
    // ----------------------------------------------------------------------
    // Implementation of the ServiceReference interface
    // ----------------------------------------------------------------------
    
    private Object callback;
    
    public Object getCallback() {
        if( callback == null ) {
            throw new NoRegisteredCallbackException();
        }
        return callback;
    }

    public Object getConversationID() {
        if( ! isConversational() ) {
            String msg = "This reference is not conversational";
            throw new IllegalStateException(msg);
        }        
        return getConversation().getConversationID();
    }

    public void setCallback(Object callback) {
        this.callback = callback;
    }

    public void setCallbackID(Object callbackID) {
        this.callbackID = callbackID;
    }

    /**
     * Set the id to associate with any conversation started through this
     * reference. If the value supplied is <code>null</code> then the id will be
     * generated by the implementation.
     *
     * @param conversationId
     *      the user-defined id to associated with a conversation
     * @throws IllegalStateException
     *      if a conversation is currently associated with this reference
     */
    public void setConversationID( Object conversationId )
    throws IllegalStateException {
        
        if( ! isConversational() ) {
            String msg = "This reference is not conversational";
            throw new IllegalStateException(msg);
        }        
        if( getConversation().getConversationID() != null ) {
            String msg =
                "A conversion has already been associated with this service "+
                "reference";
            throw new IllegalStateException(msg);
        }
        
        if( conversationId == null ) {
            conversationId = Math.random();
        }
        
        getConversation().setConversationID(conversationId);
    }
}
